<?php

/**
 * Description of SAML2AssertionChecking
 *
 * @author Luis Javier Gomez Santana <luis.gomez at rediris.es>
 */
require_once 'IAssertionChecking.php';
require_once 'AssertionPolicy.php';

class SAML2AssertionChecking implements IAssertionChecking {

    private $assertion;
    private $policy;
    private $tokenInfo;
    private $error;
    private $scope;
    private $tokenFormat;
    private $tokenEndpointURL;
    private $attributes;

    /**
     * Main Constructor
     * @param String $scope
     * @param String $dir 
     */
    public function __construct($scope, $dir = '', $urlAS = '') {
        $this->error = false;
        $this->scope = $this->cleanScope($scope);
        $this->policy = array();
        $this->tokenFormat = "%urn:mace:dir:attribute-def:eduPersonScopedAffiliation%";
        $this->tokenInfo = array();
        $this->attributes = array();
        $this->tokenEndpointURL = $urlAS;
        if ($dir == '') {
            $file = dirname(dirname(__FILE__)) . "/config/policies.xml";
        } else {
            $file = $dir . "policies.xml";
        }
        if (0 == sizeof($this->getPolicy($file))) {
            error_log("Error extracting policies");
            $this->error = true;
        }
    }

    /**
     *
     * @param String $scope
     * @return String 
     */
    private function cleanScope($scope) {
        if (strpos($scope, "?") == 0) {
            $res = $scope;
        } else {
            $res = substr($scope, 0, strpos($scope, "?"));
        }
        return $res;
    }

    /**
     *
     * @param String $assertion
     * @return boolean 
     */
    public function checkAssertion($assertion) {
        if ($this->validateAssertionFormat($assertion)) {
            $this->assertion = array();
            foreach($this->attributes as $key => $value){
                $this->assertion[$key] = $value;
            }
            $dev = false;
            if ($this->matchRules() != false) {
                $this->tokenInfo = $this->generateTokenInfo();
                if ($this->tokenInfo != NULL) {
                    $dev = true;
                }
            }
        }
        return $dev;
    }

    /**
     *
     * @param String $assertion
     * @return boolean 
     */
//file_put_contents('/tmp/prueba.txt', print_r($assertion, true), FILE_APPEND | LOCK_EX);
    private function validateAssertionFormat($assertion) {
        /*
         * contains Issuer
         * contains Conditions element with AudienceRestriction with Audience element containing an URI
         * contains Subject element 
         * contains expiry in NotOnOrAfter attribute of the Condition element or in NotOnOrAfter in SubjectConfirmationData element
         * **** contains SubjectConfirmation in Subject element with Method attribute value = urn:oasis:names:tc:SAML:2.0:cm:bearer ****
         * verify that the NotOnOrAfter instant ocurances has not passed
         * 
         */
        $res = false;
        //$sx = simplexml_load_string($assertion);
        $sx = new SimpleXMLElement($assertion, 0, false, 'urn:oasis:names:tc:SAML:2.0:assertion');
        if (isset($sx->Issuer)) { //comprobar que el Issuer es el mismo que el guardado en la configuración
            if (isset($sx->Conditions->AudienceRestriction->Audience)) {//comprobar que esto es una URI que identifica al AS o al SP de SAML, puede ser tokenendpointURL
                if (isset($sx->Subject)) {// comprobar que el subject es un authorized accesor
                    $a = (array) $sx->Conditions->attributes();
                    $a = $a['@attributes']['NotOnOrAfter'];
                    if ($this->compareWithCurrentDate($a)) {
                        $b = (array) $sx->Subject->SubjectConfirmation->attributes();
                        $b = $b['@attributes']['Method'];
                        if ((strcmp($b, "urn:oasis:names:tc:SAML:2.0:cm:bearer") == 0)) {
                            $res = true;
                        }
                    } elseif (isset($sx->Subject->SubjectConfirmation->SubjectConfirmationData)) {
                        $c = (array) $sx->Subject->SubjectConfirmation->SubjectConfirmationData->attributes();
                        $c = $c['@attributes']['Recipient'];
                        if (strcmp($c, $this->tokenEndpointURL) == 0) {
                            $d = (array) $sx->Subject->SubjectConfirmation->SubjectConfirmationData->attributes();
                            //file_put_contents('/tmp/prueba.txt', print_r($d,true), FILE_APPEND | LOCK_EX);
                            $d = $d['@attributes']['NotOnOrAfter'];
                            if ($this->compareWithCurrentDate($d)) {
                                $res = true;
                            }
                        }
                    }
                }
            }
        }
        //$sx->registerXPathNamespace('saml', 'urn:oasis:names:tc:SAML:2.0:assertion');
        foreach ($sx->AttributeStatement->Attribute as $attr) {
            $m = (array) $attr->attributes();
            $m = $m['@attributes']['Name'];
            $attr = (array) $attr;
            $this->attributes[$m] = $attr['AttributeValue'];
            //$res = $sx->xpath('//saml:Attribute[@Name="urn:mace:dir:attribute-def:eduPersonPrincipalName"]/saml:AttributeValue/text()');
        }        
        return $res;
    }

    /**
     *
     * @param String $date1 The date to compare with the current date
     */
    private function compareWithCurrentDate($date) {
        $res = true;
        //Condition NotOnOrAfter
        $splittedDate = split('T', $date);
        $ymd = $splittedDate[0];
        $ymd = split('-', $ymd);
        $asserYear = $ymd[0];
        $asserMonth = $ymd[1];
        $asserDay = $ymd[2];
        $hms = split('Z', $splittedDate[1]);
        $hms = $hms[0];
        $hms = split(':', $hms);
        $asserHours = $hms[0];
        $asserMinutes = $hms[1];
        $asserSeconds = $hms[2];
        $assertionTime = mktime($asserHours, $asserMinutes, $asserSeconds, $asserMonth, $asserDay, $asserYear);
        if (($assertionTime <= time())) {
            $res = false;
        }
        return $res;
    }

    /**
     *
     * @return Array 
     */
    private function generateTokenInfo() {
        $array_ret = array();        
        foreach ($this->tokenFormat->children() as $attribute) {
            $att = trim($attribute, "%");
            if (array_key_exists($att, $this->assertion)) {
                $array_ret[$att] = $this->assertion[$att];
            } else if (strcmp($att, 'scope') == 0) {
                $array_ret['scope'] = $this->scope;
            }
        }        
        return $array_ret;
    }

    /**
     * Function that decides if the assertion matches the policy
     * @return boolean
     */
    protected function matchRules() {
        $dev = true;
        if (sizeof($this->policy) > 0) {
            foreach ($this->policy as $pol) {
                $dev = $dev && $pol->checkPolicy($this->assertion);
            }
        } else {
            $dev = false;
        }
        return $dev;
    }

    /**
     *
     * @param String $rulesfile The path of the policy file
     * @return AssertionPolicy 
     */
    public function getPolicy($rulesfile) {
        $xml = simplexml_load_file($rulesfile);
        if (strcmp($xml->getName(), "AssertionList") == 0) {
            foreach ($xml->children() as $policy) {
                if (0 == strcmp("saml2", $policy['type'])) {
                    foreach ($policy->children() as $pol) {
                        if (0 == strcmp($this->scope, $pol['scope'])) {
                            $this->tokenFormat = $pol->TokenFormat;
                            foreach ($pol->Policy as $p) {
                                $this->policy[] = new AssertionPolicy($p);
                            }
                        }
                    }
                }
            }
        }

        return $this->policy;
    }

    public function getTokenInfo() {
        return $this->tokenInfo;
    }

    public function getError() {
        return $this->error;
    }

    public function getPolicyAttr() {
        return $this->policy;
    }

    public function getAttributes() {
        return $this->attributes;
    }

}

?>
