<?php

/**
 * Description of SIRAssertionChecking
 *
 * @author Luis Javier Gomez Santana <luis.gomez at rediris.es>
 */
require_once 'IAssertionChecking.php';
require_once 'AssertionPolicy.php';

class SIRAssertionChecking implements IAssertionChecking {

    private $assertion;
    private $policy;
    private $tokenInfo;
    private $error;
    private $scope;
    private $tokenFormat;

    /**
     * Main constructor. Load the policies from a given file. 
     * @param String $scope
     * @param String $dir 
     */
    public function __construct($scope, $dir = "") {
        $this->error = false;
        $this->scope = $this->cleanScope($scope);
        $this->policy = array();
        $this->tokenFormat = "%sho%";        
        $this->tokenInfo = array();
        if ($dir == '') {
            $file = dirname(dirname(__FILE__)) . "/config/policies.xml";
        } else {
            $file = $dir . "policies.xml";
        }
        if (0 == sizeof($this->getPolicy($file))) {
            error_log("Error extracting policies");
            $this->error = true;
        }        
    }

    /**
     *
     * @param String $scope
     * @return String 
     */
    private function cleanScope($scope) {
        if (strpos($scope, "?") == 0) {
            $res = $scope;
        } else {
            $res = substr($scope, 0, strpos($scope, "?"));
        }
        return $res;
    }

    /**
     * Function that checks if the assertion is authorized
     * @param String $userAssertion Assertion to check
     * @return boolean True if the policy matches the assertion
     */
    public function checkAssertion($userAssertion) {
        $this->assertion = $userAssertion;
        $dev = false;
        if ($this->matchRules() != false) {
            $this->tokenInfo = $this->generateTokenInfo();
            if ($this->tokenInfo != null) {
                $dev = true;
            }
        }
        return $dev;
    }

    /**
     *
     * @return Array
     */
    private function generateTokenInfo() {
        $array_ret = array();
        foreach ($this->tokenFormat->children() as $attribute) {
            $att = trim($attribute, '%');
            if (array_key_exists($att, $this->assertion))
                $array_ret[$att] = $this->assertion[$att];
            elseif (strcmp($att, 'scope') == 0)
                $array_ret['scope'] = $this->scope;
        }        
        return $array_ret;
    }

    /**
     * Function that decides if the assertion matches the policy
     * @return boolean
     */
    protected function matchRules() {        
        $dev = true;        
        if (sizeof($this->policy) > 0) {
            foreach ($this->policy as $pol) {
                $dev = $dev && $pol->checkPolicy($this->assertion);
            }
        } else {
            $dev = false;
        }
        return $dev;
    }

    /**
     *
     * @param String $rulesfile The path of the policy file
     * @return AssertionPolicy 
     */
    public function getPolicy($rulesfile) {
        $xml = simplexml_load_file($rulesfile);
        if (strcmp($xml->getName(), "AssertionList") == 0) {
            foreach ($xml->children() as $policy) {
                if (0 == strcmp("papi", $policy['type'])) {
                    foreach ($policy->children() as $pol) {
                        if (0 == strcmp($this->scope, $pol['scope'])) {                                                        
                            $this->tokenFormat = $pol->TokenFormat;                            
                            foreach ($pol->Policy as $p) {
                                $this->policy[] = new AssertionPolicy($p);                                
                                
                            }
                        }
                    }
                }
            }
        }

        return $this->policy;
    }

    public function getTokenInfo() {
        return $this->tokenInfo;
    }

    public function getError() {
        return $this->error;
    }

    public function getPolicyAttr() {
        return $this->policy;
    }

}

?>
