<?php

/**
 * Description of LoadConfigJSON
 *
 * @author Luis Javier Gomez Santana <luis.gomez at rediris.es>
 */
class LoadConfigJSON {

    //Assertion Types
    protected $error_type;
    protected $debug_active_client;
    // Client Credentials
    protected $client_id;
    protected $client_secret;
    //Access Token request Data
    protected $assertion_type;
    protected $scope;
    protected $as;
    protected $grant_type;
    //Resource Token request Data
    protected $request_type;
    protected $rs;
    protected $format_classes;
    protected $format_archive_names;
    protected $as_token_type;
    protected $storage_type;

    /**
     *
     * @param String $file 
     */
    public function __construct($file) {
        $this->format_classes = array();
        $this->format_archive_names = array();
        $this->loadFile($file);
    }

    /**
     *
     * @param String $file 
     */
    public function loadFile($file) {
        $json_cfg = file_get_contents($file);
        $json_cfg_array = json_decode($json_cfg, true);
        //AS config
        $this->assertion_type = $json_cfg_array['OAuthClient']['AuthServerConfig']['AssertionType'];
        $this->grant_type = $json_cfg_array['OAuthClient']['AuthServerConfig']['GrantType'];
        $this->as = $json_cfg_array['OAuthClient']['AuthServerConfig']['AuthServerURL'];
        //$this->as_token_type = $json_cfg_array['OAuthClient']['AuthServerConfig']['AuthServerTokenType'];
        //RS config
        $this->request_type = $json_cfg_array['OAuthClient']['ResServerConfig']['RequestType'];
        $this->rs = $json_cfg_array['OAuthClient']['ResServerConfig']['ResServerURL'];
        foreach ($json_cfg_array['OAuthClient']['ResServerConfig']['ResponseFormats'] as $res_format) {
            $id = $res_format['attributes']['id'];
            if (isset($res_format['FormatClass']))
                $this->format_classes[$id] = $res_format['FormatClass'];
            if (isset($res_format['FormatFile']))
                $this->format_archive_names[$id] = $res_format['FormatFile'];
        }
        //Client config
        $this->scope = $json_cfg_array['OAuthClient']['ClientConfig']['DefaultScope'];
        $this->client_id = $json_cfg_array['OAuthClient']['ClientConfig']['ClientID'];
        $this->client_secret = $json_cfg_array['OAuthClient']['ClientConfig']['ClientSecret'];
        $this->error_type = $json_cfg_array['OAuthClient']['ClientConfig']['ErrorResponseType'];
        $this->debug_active_client = $json_cfg_array['OAuthClient']['ClientConfig']['DebugActive'];
        $this->storage_type = $json_cfg_array['OAuthClient']['ClientConfig']['StorageType'];
    }

    public function getFormatClass($scope) {
        return $this->format_classes[$scope];
    }

    public function getFormatArchiveName($scope) {
        return $this->format_archive_names[$scope];
    }

    public function hasFormatClass($scope) {
        return array_key_exists($scope, $this->format_classes);
    }

    public function hasFormatArchiveName($scope) {
        return array_key_exists($scope, $this->format_archive_names);
    }

    public function get_error_type() {
        return $this->error_type;
    }

    public function get_debug_active_client() {
        return $this->debug_active_client;
    }

    public function get_client_id() {
        return $this->client_id;
    }

    public function get_client_secret() {
        return $this->client_secret;
    }

    public function get_assertion_type() {
        return $this->assertion_type;
    }

    public function get_scope() {
        return $this->scope;
    }

    public function get_as() {
        return $this->as;
    }

    public function get_grant_type() {
        return $this->grant_type;
    }

    public function get_request_type() {
        return $this->request_type;
    }

    public function get_rs() {
        return $this->rs;
    }

    public function get_storType() {
        return $this->storage_type;
    }

}

?>
