<?php

/**
 * Description of OAuthRS
 *
 * @author Luis Javier Gomez Santana <luis.gomez at rediris.es>
 */
include_once('ErrorList.php');
include_once('AuthServerList.php');
include_once('LoadResourceConfig.php');

include_once('ResServerJWT.php');
include_once '/Users/luija/Sites/html/phpPoA-2.4/PoA.php';

class OAuthRS {

    protected $error;
    protected $debug_active;
    protected $authservers;
    protected $errors;
    protected $scope;
    protected $extra;
    protected $resource;
    protected $token;
    protected $token_info;
    protected $token_format;
    protected $config_dir;

    /**
     *
     * @param String $dir 
     */
    public function __construct($dir= "") {
        if (0 == strcmp($dir, "")) {
            $this->config_dir = dirname(dirname(__FILE__)) . "/config/";
        } else {
            $this->config_dir = $dir;
            $last_char = substr($dir, strlen($dir) - 1);
            if (strcmp("/", $last_char) != 0) {
                $this->config_dir .= "/";
            }
        }
        $this->authservers = new AuthServerList($this->config_dir);
        $this->error = null;
        $this->errors = new ErrorList($this->config_dir);
        $this->debug_active = false;
        $this->scope = null;
        $this->extra = array();
        $this->token = null;
        //$this->token_info = "";
        $this->token_info = array();
        $this->token_format = array();
        $this->resource = null;
    }

    /**
     *
     * @param String $string 
     */
    private function error($string) {
        if ($this->debug_active) {
            error_log("OAuth_RS: " . $string);
        }
    }

    /**
     * 
     */
    public function manageRequest() {
        $this->error("manageRequest");
        if ($this->isValidFormatRequest()) {
            if ($this->isValidToken()) {
                $this->manageRSResponse();
            } else {
                $this->manageRSErrorResponse();
            }
        }
        if ($this->error != null) {
            $this->manageRSErrorResponse();
        }
    }

    /**
     *
     * @return boolean 
     */
    private function isValidFormatRequest() {
        $this->error("isValidFormatRequest");
        $dev = false;
        if (array_key_exists("CONTENT_TYPE", $_SERVER) && isset($_POST) && sizeof($_POST) > 0
                && (strcmp($_SERVER['CONTENT_TYPE'], "application/x-www-form-urlencoded") == 0)) {
            $dev = $this->isValidFormatGETorPOSTRequest($_POST);
        } else if (isset($_GET) && (sizeof($_GET) > 0)) {
            $dev = $this->isValidFormatGETorPOSTRequest($_GET);
        } else {
            $dev = $this->isValidFormatHeaderRequest(apache_request_headers());
        }
        return $dev;
    }

    /**
     *
     * @param Array $request
     * @return boolean 
     */
    private function isValidFormatGETorPOSTRequest($request) {
        $this->error("isValidGetorPOSTRequest");
        $dev = false;
        foreach ($request as $k => $val) {
            if (strcmp($k, "access_token") == 0) {
                $this->token = $val;
                $dev = true;
            } else {
                $this->extra[$k] = $val;
            }
        }
        if (!$dev) {
            $this->error .= "invalid_request";
        }
        return $dev;
    }

    /**
     *
     * @param Array $headers
     * @return boolean 
     */
    private function isValidFormatHeaderRequest($headers) {
        $this->error("isValidHeaderRequest");
        $dev = false;
        //var_dump($headers);
        if (array_key_exists("Authorization", $headers)) {
            $value = $headers['Authorization'];
            $array = explode("Bearer ", $value);
            if (count($array) < 2) {
                $this->error = "invalid_request";
            } else {
                //$array = explode(",", $array[1]);
                $this->token = $array[1]; //array[0];
                if (count($array) > 1) {
                    $filtro = '/(.*)="(.*?)"/';
                    foreach ($array as $value) {
                        if (1 == preg_match_all($filtro, $value, $out)) {
                            array_push($this->extra, array($out[1][0] => $out[2][0]));
                        }
                    }
                }
                if (isset($_REQUEST)) {
                    foreach ($_REQUEST as $id => $req) {
                        $this->extra[$id] = $req;
                        $this->error(print_r($this->extra, 1));
                    }
                }
            }
        }
        if ($this->token == null) {
            $this->error = "invalid_request";
        } else {
            $dev = true;
        }
        return $dev;
    }

    /**
     *
     * @param String $scope 
     */
    private function processScope($scope) {
        $s = explode("?", $scope);
        $this->scope = $s[0];
        if (isset($s[1])) {
            $atts = explode("&", $s[1]);
            foreach ($atts as $att) {
                $aux = explode("=", $att);
                $this->extra[$aux[0]] = $aux[1];
            }
        }
    }

    /**
     *
     * @return boolean 
     */
    private function isValidToken() {

        $res = true;
        $res_jwt = new ResServerJWT("/Users/luija/Sites/html/oauth2libv14/oauth_server/src/publickey.pem");
        $claims = $res_jwt->decode($this->token);
        if (!$claims == NULL) {
            $this->token_info = $claims['token_info'];
            $this->scope = $claims['scope'];
            $this->processScope($this->scope);
            $this->createResource($this->scope);
            $info = $this->addTokenInfo($this->token_info);
            $info = $this->token_info;
            //file_put_contents('/tmp/prueba2.txt', $this->scope, FILE_APPEND | LOCK_EX);
            if (!$this->authservers->checkAuthzKey($claims['authzID'])) {
                $this->error = "Invalid AS key";
                $res = false;
            }
            if (!microtime(true) > $claims['exp']) {
                $this->error = 'expired_token';
                $res = false;
            }
            //if (null==$info) {
            if (count($this->token_format) != count($this->token_info)) {
                $this->error = "INSUFFICIENT_SCOPE";
            }
        }else
            $res = false;
        return $res;
    }

    /**
     *
     * @param Array $token_info
     * @return Array 
     */
    private function addTokenInfo($token_info) {
        $this->error('addTokenInfo');
        $dev = null;
        //$token_info_attrs = explode("&&",$this->token_info);
        if (count($this->token_format) == count($token_info)) {
            $dev = array_combine($this->token_format, $token_info);
        }
        //  $dev = $this->resource->checkScope($this->scope, $token_info);
        return $dev;
    }

    /**
     * 
     */
    private function manageRSErrorResponse() {
        if (0 == strcmp($this->error, "insufficient_scope")) {
            header("HTTP/1.0 403 Forbidden");
            header("Content-Type: application/json");
            header("Cache-control:no-store");
            echo json_encode(array("error" => $this->error, "scope" => $this->scope));
        } else if (0 == strcmp($this->error, "invalid_request")) {
            header("HTTP/1.0 400 Bad Request");
            header("Content-Type: application/json");
            header("Cache-control:no-store");
            echo json_encode(array("error" => $this->error));
        } else if (0 == strcmp($this->error, "invalid_token")) {
            header("HTTP/1.0 401 Unauthorized");
            header("Content-Type: application/json");
            header("Cache-control:no-store");
            echo json_encode(array("error" => $this->error));
        } else {
            //expired-token
            header("HTTP/1.0 401 Unauthorized");
            header("Content-Type: application/json");
            header("Cache-control:no-store");
            echo json_encode(array("error" => $this->error));
        }
    }

    /**
     * Function that returns the resource, making use of the Resource Class deployed in the server.
     */
    private function manageRSResponse() {
        if ($this->resource != null) {
            $res = $this->resource->getResource($this->scope, $this->extra);
            if ($res == null) {
                $this->error = "invalid_token";
                $this->manageRSErrorResponse();
            } else {
                if ($this->resource->hasHeader()) {
                    foreach ($this->resource->getHeader() as $line) {
                        header($line);
                    }
                }
                echo $res;
            }
        } else {
            $this->error = "own_invalid_token";
            $this->manageRSErrorResponse();
        }
    }

    /**
     * 
     */
    private function createResource() {
        $this->error("createResource");
        $conf = new LoadResourceConfig($this->config_dir);
        if ($conf->hasClass($this->scope)) {
            $class = $conf->getClass($this->scope);
            if ($conf->hasArchiveName($this->scope)) {
                include_once 'resources/' . $conf->getArchiveName($this->scope);
                $reflect = new ReflectionClass($class);
                $this->resource = $reflect->newInstance();
                if ($conf->hasArchiveName($this->scope)) {
                    $this->token_format = $conf->getTokenFormats($this->scope);
                } else {
                    $this->error = "invalid-resource-configuration3";
                }
            } else {
                $this->error = "invalid-resource-configuration";
            }
        } else {
            $this->error = "invalid-resource-configuration2";
        }
    }

}

?>
